@@ -7,17 +7,18 @@ use utf8;
 use version;
 
 use Carp;
-use DR::Money;
 use Mail::RFC822::Address;
 use DateTime;
 use DateTime::Format::DateParse;
-use POSIX                           qw(strftime);
-use Mojo::JSON                      qw(from_json);
 use List::MoreUtils                 qw(any firstval);
 
+use Mojo::URL;
+use Mojo::JSON;
+use Mojo::DOM;
+
 use Mojolicious::Plugin::Vparam::Address;
 
-our $VERSION = '1.1';
+our $VERSION = '1.12';
 
 =encoding utf-8
 
@@ -29,17 +30,47 @@ Mojolicious::Plugin::Vparam - Mojolicious plugin validator for GET/POST data.
 
 Features:
 
-    * Simple syntax or full featured
-    * Many predefined types
-    * Support arrays of values
-    * Support HTML checkbox as bool
-    * Validate all parameters at once and get hash to simple use in any Model
-    * Manage valudation errors
-    * L<# Mojo::Validator::Validation> integration
+=over
 
-This module use simple paramters types str, int, email, bool, etc. to validate.
-Instead of many other modules you not need add specific validation subs or
-rules. Just set parameter type. But if you want sub or rule you can do it too.
+=item *
+
+Simple syntax or full featured
+
+=item *
+
+Many predefined types
+
+=item *
+
+Filters complementary types
+
+=item *
+
+Support arrays of values
+
+=item *
+
+Support HTML checkbox as bool
+
+=item *
+
+Validate all parameters at once and get hash to simple use in any Model
+
+=item *
+
+Manage validation errors
+
+=item *
+
+Full Mojolicious::Validator::Validation integration
+
+=back
+
+This module use simple parameters types B<str>, B<int>, B<email>, B<bool>,
+etc. to validate.
+Instead of many other modules you mostly not need add specific validation
+subs or rules.
+Just set parameter type. But if you want sub or regexp you can do it too.
 
 =head1 SYNOPSIS
 
@@ -48,11 +79,12 @@ rules. Just set parameter type. But if you want sub or rule you can do it too.
 
     # Use in controller
     $login = $self->vparam(login    => 'str');
-    $passw = $self->vparam(password => 'password');
+    $passw = $self->vparam(password => 'password', size => [8, 100]);
+    $email = $self->vparam(email    => 'email', optional => 1);
 
 =head1 METHODS
 
-=head2 vparam ...
+=head2 vparam
 
 Get one parameter. By default parameter is required.
 
@@ -64,9 +96,9 @@ Get one parameter. By default parameter is required.
     # Or more simple syntax
     $param2 = $self->vparam(count => 'int', default => 1);
 
-=head2 vparms ...
+=head2 vparams
 
-Get many parameters as hash. By default parameters are required.
+Get many parameters as hash. By default all parameters are required.
 
     %params = $self->vparams(
         # Simple syntax
@@ -86,14 +118,37 @@ Get many parameters as hash. By default parameters are required.
         isa         => { type => 'bool', default => 0 },
     );
 
-=head2 vsort ...
+=head2 vsort
 
-Like I<vparams> but add some keys to simple use with tables.
+Like I<vparams> but add some keys to simple use with tables. Example:
 
-    %filters = $self->vsort(
+    # HTML - table with controls and filters
+    Order by:
+    <select name="oby">
+        <option value="0">Name</option>
+        <option value="1">Date</option>
+    </select>
+    Order direction:
+    <select name="ods">
+        <option value="asc">Low to High</option>
+        <option value="desc">High to Low</option>
+    </select>
+    Count per page:
+    <select name="rws">
+        <option value="10">10</option>
+        <option value="100">100</option>
+    </select>
+    Filter by name:
+    <input type="text" name="name" value="">
+    Any other filters ...
+
+
+    # Controller
+    %params = $self->vsort(
         -sort       => ['name', 'date', ...],
 
         # next as vparams
+        name        => 'text',
         ...
     );
 
@@ -118,9 +173,14 @@ If you set undef then parameter is not apply.
 
 Column number for sorting. Default: 1 - in many cases first database
 column is primary key.
-If you set undef then parameter is not apply.
 
 You can set different name by I<vsort_oby> config parameter.
+If you set undef then parameter is not apply.
+
+Value of B<oby> The value will be automatically mapped to the column name
+using the L</-sort> attribute.
+Also, the value will be checked for proper mapping.
+So you do not need to worry about it.
 
 =item ods
 
@@ -135,7 +195,35 @@ If you set undef then parameter is not apply.
 
 Get parameter error string. Return 0 if no error.
 
+    # Get error
     print $self->verror('myparam') || 'Ok';
+    # Set error
+    $self->verror('myparam', {message => 'Error message'})
+
+=head2 vclass $name, @classes
+
+Get classes for invalid input. Return empty string if no error.
+
+    # Form example
+    <input name="myparam" class="<%= vclass 'myparam' %>">
+    # Return next code for invalid I<myparam>:
+    # <input name="myparam" class="field-with-error">
+
+You can set additional I<@classes> to set if field invalid.
+
+=head2 vvalue $name, $default
+
+Get raw input value after validation. Return I<$default> value or empty
+string before validation.
+
+    # Form example:
+    <input name="myparam" value="<%= vvalue 'myparam' %>">
+
+    # Return next code if user just open form without submit and validation:
+    # <input name="myparam" value="">
+
+    # Then user submit form and you validate id. For example user submit "abc":
+    # <input name="myparam" value="abc">
 
 =head2 verrors
 
@@ -151,15 +239,88 @@ Return erorrs count in scalar context. In list context return erorrs hash.
 
 Set new type $name if defined %opts. Else return type $name definition.
 
+    # Get type
+    $self->vtype('mytype');
+
+    # Set type
+    # pre   - get int
+    # valid - check for not empty
+    # post  - force number
+    $self->vtype('mytype',
+        pre     => sub {
+            my ($self, $param) = @_;
+            return int $param // '';
+        },
+        valid   => sub {
+            my ($self, $param) = @_;
+            return length $param ? 0 : 'Invalid'
+        },
+        post    => sub {
+            my ($self, $param) = @_;
+            return 0 + $param;
+        }
+    );
+
+=head2 vfilter $name, &sub
+
+Set new filter $name if defined %opts. Else return filter $name definition.
+
+    # Get filter
+    $self->vfilter('myfilter');
+
+    # Set filter
+    $self->vfilter('myfilter', sub {
+        my ($self, $param, $expression) = @_;
+        return $param eq $expression ? 0 : 'Invalid';
+    });
+
+Filter sub must return 0 if parameter value is valid. Or error string if not.
+
+=head1 SIMPLE SYNTAX
+
+You can use the simplified syntax instead of specifying the type,
+simply by using an expression instead.
+
+=over
+
+=item I<REGEXP>
+
+Apply as L</regexp> filter. No type verification, just match.
+
+    $self->vparam(myparam => qr{^(abc|cde)$});
+
+=item I<CODE> $mojo, $value
+
+Apply as L</post> function. You need manual verify and set error.
+
+    $self->vparam(myparam => sub { $_[1] && $_[1] eq 'good' ? 1 : 0 });
+
+=item I<ARRAY>
+
+Apply as L</in> filter. No type verification, just match.
+
+    $self->vparam(myparam => [qw(abc cde)]);
+
+=back
+
 =head1 CONFIGURATION
 
 =over
 
+=item class
+
+CSS class for invalid parameters. Default: field-with-error.
+
 =item types
 
 You can simple add you own types.
 Just set this parameters as HashRef with new types definition.
 
+=item filters
+
+You can simple add you own filters.
+Just set this parameters as HashRef with new filters definition.
+
 =item vsort_page
 
 Parameter name for current page number in I<vsort>. Default: page.
@@ -223,7 +384,7 @@ Minimum password length. Default: 8.
 
 =item mojo_validator
 
-Enable L<# Mojo::Validator::Validation> integration.
+Enable L<Mojolicious::Validator::Validation> integration.
 
 =back
 
@@ -239,7 +400,7 @@ List of supported types:
 
 =item int
 
-Signed integer
+Signed integer. Use L</min> filter for unsigned.
 
 =cut
 
@@ -251,7 +412,7 @@ sub _check_int($) {
 
 =item numeric or number
 
-Signed number
+Signed number. Use L</min> filter for unsigned.
 
 =cut
 
@@ -264,7 +425,7 @@ sub _check_numeric($) {
 
 =item money
 
-Get L<DR::Money> object for proper money operations.
+Get money. Use L</min> filter for unsigned.
 
 =cut
 
@@ -383,7 +544,51 @@ sub _check_time($) {
 
 Get full date and time. Parsed from many formats.
 See I<datetime> configuration parameter for result format.
-See L<DateTime::Format::DateParse> and even more.
+
+Input formats:
+
+=over
+
+=item *
+
+Timestamp.
+
+=item *
+
+Relative from now in format C<[+-] DD HH:MM:SS>. First sign required.
+
+=over
+
+=item *
+
+Minutes by default. Example: C<+15> or C<-6>.
+
+=item *
+
+Minutes and seconds. Example: C<+15:44>.
+
+=item *
+
+Hours. Example: C<+3:15:44>.
+
+=item *
+
+Days. Example: C<+8 3:15:44>.
+
+=back
+
+Values are given in arbitrary range.
+For example you can add 400 minutes and 300 seconds: C<+400:300>.
+
+=item *
+
+All that can be obtained L<DateTime::Format::DateParse>.
+
+=item *
+
+Russian date format like C<DD.MM.YYYY>
+
+=back
 
 =cut
 
@@ -402,6 +607,22 @@ You need always set default value to supress error if checkbox not checked:
 
     $self->vparam(mybox => 'bool', default => 0);
 
+Valid values are:
+
+=over
+
+=item
+
+I<TRUE> can be 1, yes, true, ok
+
+=item
+
+I<FALSE> can be 0, no, false, fail
+
+=back
+
+Other values get error.
+
 =cut
 
 sub _check_bool($) {
@@ -439,7 +660,7 @@ sub _check_url($) {
 
 =item phone
 
-Phone in international format.
+Phone in international format. Support B<wait>, B<pause> and B<additional>.
 
 You can set default country I<phone_country> and region I<phone_country> codes.
 Then you users can input shortest number.
@@ -597,20 +818,22 @@ You can set a simple mode as in example or full mode. Full mode keys:
 
 Default value. Default: undef.
 
-
-=item regexp $mojo, $regexp
-
-Valudator regexp by $regexp.
+    # Supress myparam to be undefined and error
+    $self->vparam(myparam => 'str', default => '');
 
 =item pre $mojo, &sub
 
 Incoming filter sub. Used for primary filtration: string length and trim, etc.
 Result will be used as new param value.
 
+Usually, if you need this attribute, you need to create a new type.
+
 =item valid $mojo, &sub
 
 Validation sub. Return 0 if valid, else string of error.
 
+Usually, if you need this attribute, you need to create a new type.
+
 =item post $mojo, &sub
 
 Out filter sub. Used to modify value for use in you program. Usually used to
@@ -619,11 +842,11 @@ Result will be used as new param value.
 
 =item type
 
-Parameter type. If set then some filters will be apply.
+Parameter type. If set then some filters will be apply. See L</TYPES>.
 
-    See L<TYPES>
+    $self->vparam(myparam => 'datetime');
 
-After apply all type filters, regexp and post filters will be apply too if set.
+After the application of the type used filters.
 
 =item array
 
@@ -637,6 +860,7 @@ You can force values will arrays by B<@> prefix or B<array[...]>.
     $param3 = $self->vparam(array3 => 'int', array => 1);
 
     # The array will come if more than one value incoming
+    # Example: http://mysite.us?array4=123&array4=456...
     $param4 = $self->vparam(array4 => 'int');
 
 =item optional
@@ -646,6 +870,7 @@ set "optional".
 Then true and value is not passed validation don`t set verrors.
 
     # Simple vparam
+    # myparam is undef but no error.
     $param6 = $self->vparam(myparam => 'int', optional => 1);
 
     # Set one in vparams
@@ -657,78 +882,64 @@ Then true and value is not passed validation don`t set verrors.
     %params = $self->vparams(
         -optional   => 1,
         param1      => 'int',
-        param2      => 'int',
+        param2      => 'str',
     );
 
-=cut
+=item skip
 
-=item min
+So as not to smear the validation code you can use the I<skip> parameter
+to skip on the condition.
+This attribute is useful for controlling access to the form fields.
 
-Check minimum parameter value.
+    # This example don`t get param1 in production mode.
 
-=cut
+    # HTML
+    % unless( $self->app->mode eq 'production' ) {
+        %= number_field 'param1'
+    % }
 
-sub _min($$) {
-    my ($value, $min) = @_;
-    return sprintf "Value should not be less than %s", $min
-        unless $value >= $min;
-    return 0;
-}
+    # Simple flag
+    $param1 = $self->vparam(
+        param1      => 'int', skip => $self->app->mode eq 'production',
+    );
 
-=item max
+    # Same as by use sub.
+    $param1 = $self->vparam(
+        param1      => 'int', skip => sub { $_[0]->app->mode eq 'production' },
+    );
 
-Check maximum parameter value.
+If you use sub then first parameter is controller.
 
-=cut
+=item jpath
 
-sub _max($$) {
-    my ($value, $max) = @_;
-    return sprintf "Value should not be greater than %s", $max
-        unless $value <= $max;
-    return 0;
-}
+If you POST data not form but raw JSON you can user JSON Pointer selectors
+to get and validate parameters.
 
-=item range
+    # POST data contains:
+    # {"point":{"address":"some", "lon": 45.123456, "lat": 38.23452}}
 
-Check parameter value to be in range.
+    %opts = $self->vparams(
+        address => { type => 'str', jpath => '/point/address' },
+        lon     => { type => 'lon', jpath => '/point/lon' },
+        lat     => { type => 'lat', jpath => '/point/lat' },
+    );
 
-=cut
+=item cpath
 
-sub _range($$$) {
-    my ($value, $minimum, $maximum) = @_;
+Same as jpath but parse XML/HTML sing CSS selectors.
 
-    my $min = _min $value => $minimum;
-    return $min if $min;
+    # POST data contains:
+    # <Point>
+    #    <Address>some</Address>
+    #    <Lon>45.123456</Lon>
+    #    <Lat>38.23452</Lat>
+    # </Point>
 
-    my $max = _max $value => $maximum;
-    return $max if $max;
-
-    return 0;
-}
-
-# Regexp
-sub _like($$) {
-    my ($value, $re) = @_;
-    return 'Value not defined'      unless defined $value;
-    return 'Wrong format'           unless $value =~ $re;
-    return 0;
-}
-
-=item in
-
-Check parameter value to be in list of defined values.
-
-=cut
-
-sub _in($$) {
-    my ($value, $array) = @_;
-    confess 'Not ArrayRef'          unless 'ARRAY' eq ref $array;
-
-    return 'Value not defined'      unless defined $value;
-    return 'Wrong value'            unless any {$value eq $_} @$array;
-
-    return 0;
-}
+    %opts = $self->vparams(
+        address => { type => 'str', jpath => 'Point > Address' },
+        lon     => { type => 'lon', jpath => 'Point > Lon' },
+        lat     => { type => 'lat', jpath => 'Point > Lat' },
+    );
 
 =back
 
@@ -746,7 +957,127 @@ from set too much or too low column number.
 
 =item -optional
 
-Set default optional flag for all params in L<vparams> and I<vsort>;
+Set default optional flag for all params in L</vparams> and L</vsort>.
+
+=back
+
+=cut
+
+=head1 FILTERS
+
+Filters are used in conjunction with types for additional verification.
+
+=cut
+
+=over
+
+=item min
+
+Check minimum parameter value.
+
+    # Error if myparam less than 10
+    $self->vparam(myparam => 'int', min => 10);
+
+=cut
+
+sub _min($$) {
+    my $numeric = _check_numeric $_[0];
+    return $numeric if $numeric;
+
+    return sprintf "Value should not be greater than %s", $_[1]
+        unless $_[0] >= $_[1];
+    return 0;
+}
+
+=item max
+
+Check maximum parameter value.
+
+    # Error if myparam greater than 100
+    $self->vparam(myparam => 'int', max => 100);
+
+=cut
+
+sub _max($$) {
+    my $numeric = _check_numeric $_[0];
+    return $numeric if $numeric;
+
+    return sprintf "Value should not be less than %s", $_[1]
+        unless $_[0] <= $_[1];
+    return 0;
+}
+
+=item range
+
+Check parameter value to be in range.
+
+    # Error if myparam less than 10 or greater than 100
+    $self->vparam(myparam => 'int', range => [10, 100]);
+
+=cut
+
+sub _range($$$) {
+    my $min = _min $_[0] => $_[1];
+    return $min if $min;
+
+    my $max = _max $_[0] => $_[2];
+    return $max if $max;
+
+    return 0;
+}
+
+=item regexp
+
+Check parameter to be match for regexp
+
+    # Error if myparam not equal "abc" or "cde"
+    $self->vparam(myparam => 'str', regexp => qr{^(abc|cde)$});
+
+=cut
+
+sub _like($$) {
+    return 'Value not defined'      unless defined $_[0];
+    return 'Wrong format'           unless $_[0] =~ $_[1];
+    return 0;
+}
+
+=item in
+
+Check parameter value to be in list of defined values.
+
+    # Error if myparam not equal "abc" or "cde"
+    $self->vparam(myparam => 'str', in => [qw(abc cde)]);
+
+=cut
+
+sub _in($$) {
+    confess 'Not ArrayRef'          unless 'ARRAY' eq ref $_[1];
+
+    return 'Value not defined'      unless defined $_[0];
+    return 'Wrong value'            unless any {$_[0] eq $_} @{$_[1]};
+
+    return 0;
+}
+
+=item size
+
+Check maximum length in utf8.
+
+    # Error if value is an empty string
+    $self->vparam(myparam => 'str', size => [1, 100]);
+
+=cut
+
+sub _size($$$) {
+    my ($value, $min, $max) = @_;
+    return 'Value is not defined'       unless defined $value;
+    return 'Value is not set'           unless length  $value;
+    return sprintf "Value should not be less than %s", $min
+        unless $min <= length $value;
+    return sprintf "Value should not be longer than %s", $max
+        unless $max >= length $value;
+    return 0;
+}
 
 =back
 
@@ -798,24 +1129,44 @@ sub _parse_date($;$) {
     my $dt;
 
     if( $str =~ m{^\d+$} ) {
-        $dt = DateTime->from_epoch( epoch => int $str );
-    } elsif( $str =~ m{^[\+\-]\d+$} ) {
-        my $minutes = int $str;
-        $dt = DateTime->now();
-        $dt->add(minutes => $minutes);
+        $dt = DateTime->from_epoch( epoch => int $str, time_zone => 'local' );
+    } elsif( $str =~ m{^[+-]} ) {
+        my @relative = $str =~ m{
+            ^([+-])             # sign
+            \s*
+            (?:(\d+)\s+)?       # days
+            (?:(\d+):)??        # hours
+            (\d+)               # minutes
+            (?::(\d+))?         # seconds
+        $}x;
+        $dt = DateTime->now(time_zone => 'local');
+        my $sub = $relative[0] eq '+' ? 'add' : 'subtract';
+        $dt->$sub(days      => int $relative[1])    if defined $relative[1];
+        $dt->$sub(hours     => int $relative[2])    if defined $relative[2];
+        $dt->$sub(minutes   => int $relative[3])    if defined $relative[3];
+        $dt->$sub(seconds   => int $relative[4])    if defined $relative[4];
     } else {
         # RU format
-        $str =~ s{^(\d{1,2})\.(\d{1,2})\.(\d{4})(.*)$}{$3-$2-$1$4};
+        if( $str =~ s{^(\d{1,2})\.(\d{1,2})\.(\d{1,4})(.*)$}{$3-$2-$1$4} ) {
+            my $cur_year = DateTime->now(time_zone => 'local')->strftime('%Y');
+            my $cur_len  = length( $cur_year ) - 1;
+            # Less digit year
+            if( my ($year) = $str =~ m{^(\d{1,$cur_len})-} ) {
+                $str = substr($cur_year, 0, 4 - length($year)) . $str;
+            }
+        }
         # If looks like time add it
-        $str = DateTime->now->strftime('%F ') . $str if $str =~ m{^\s*\d{2}:};
+        $str = DateTime->now(time_zone => 'local')->strftime('%F ') . $str
+            if $str =~ m{^\d{2}:};
 
         $dt = eval { DateTime::Format::DateParse->parse_datetime( $str ); };
-        return undef if !$dt;
         return undef if $@;
     }
 
-    # Set timezone
-    $tz //= strftime '%z', localtime;
+    return undef unless $dt;
+
+    # Always local timezone
+    $tz //= DateTime->now(time_zone => 'local')->strftime('%z');
     $dt->set_time_zone( $tz );
 
     return $dt;
@@ -834,7 +1185,13 @@ sub _parse_json($) {
     return undef unless defined $str;
     return undef unless length  $str;
 
-    my $data = eval{ from_json $str };
+    my $data = eval{
+        if( version->new($Mojolicious::VERSION) < version->new(5.54) ) {
+            return Mojo::JSON->new->decode( $str );
+        } else {
+            return Mojo::JSON::decode_json( $str );
+        }
+    };
     warn $@ and return undef if $@;
 
     return $data;
@@ -869,7 +1226,9 @@ sub register {
 
     $conf                   ||= {};
 
+    $conf->{class}          ||= 'field-with-error';
     $conf->{types}          ||= {};
+    $conf->{filters}        ||= {};
 
     $conf->{vsort_page}     ||= 'page';
     $conf->{vsort_rws}      ||= 'rws';
@@ -881,9 +1240,9 @@ sub register {
     $conf->{phone_country}  //= '';
     $conf->{phone_region}   //= '';
 
-    $conf->{date}           //= '%F';
-    $conf->{time}           //= '%T';
-    $conf->{datetime}       //= '%F %T %z';
+    $conf->{date}           = '%F'          unless exists $conf->{date};
+    $conf->{time}           = '%T'          unless exists $conf->{time};
+    $conf->{datetime}       = '%F %T %z'    unless exists $conf->{datetime};
 
     $conf->{optional}       //= 0;
 
@@ -891,7 +1250,10 @@ sub register {
 
     $conf->{password_min}   //= 8;
 
-    $conf->{mojo_validator} //= 1;
+    # Enable Mojolicious::Validator::Validation integration if available
+    $conf->{mojo_validator} //=
+        version->new($Mojolicious::VERSION) < version->new(4.42) ? 0 : 1;
+
 
     $conf->{types} = {
         # Numbers
@@ -908,18 +1270,22 @@ sub register {
         money       => {
             pre     => sub{ _parse_number   $_[1] },
             valid   => sub{ _check_money    $_[1] },
+            post    => sub{ defined         $_[1] ? 0.0 + $_[1] : undef },
         },
         percent     => {
             pre     => sub{ _parse_number   $_[1] },
             valid   => sub{ _check_percent  $_[1] },
+            post    => sub{ defined         $_[1] ? 0.0 + $_[1] : undef },
         },
         lon         => {
             pre     => sub{ _parse_number   $_[1] },
             valid   => sub{ _check_lon      $_[1] },
+            post    => sub{ defined         $_[1] ? 0.0 + $_[1] : undef },
         },
         lat         => {
             pre     => sub{ _parse_number   $_[1] },
             valid   => sub{ _check_lat      $_[1] },
+            post    => sub{ defined         $_[1] ? 0.0 + $_[1] : undef },
         },
 
         # Text
@@ -945,7 +1311,7 @@ sub register {
             valid   => sub { _check_date        $_[1] },
             post    => sub {
                 return unless defined $_[1];
-                return $conf->{date}
+                return $conf->{date} && ref $_[1]
                     ? $_[1]->strftime( $conf->{date} )
                     : $_[1];
             },
@@ -955,7 +1321,7 @@ sub register {
             valid   => sub { _check_time        $_[1] },
             post    => sub {
                 return unless defined $_[1];
-                return $conf->{time}
+                return $conf->{time} && ref $_[1]
                     ? $_[1]->strftime( $conf->{time} )
                     : $_[1];
             },
@@ -965,7 +1331,7 @@ sub register {
             valid   => sub { _check_datetime    $_[1] },
             post    => sub {
                 return unless defined $_[1];
-                return $conf->{datetime}
+                return $conf->{datetime} && ref $_[1]
                     ? $_[1]->strftime( $conf->{datetime} )
                     : $_[1];
             },
@@ -1023,13 +1389,31 @@ sub register {
     # Aliases
     $conf->{types}{number} = $conf->{types}{numeric};
 
+    $conf->{filters} = {
+        regexp      => sub { _like      $_[1], $_[2] },
+        in          => sub { _in        $_[1], $_[2] },
+        min         => sub { _min       $_[1], $_[2] },
+        max         => sub { _max       $_[1], $_[2] },
+        range       => sub { _range     $_[1], $_[2][0], $_[2][1] },
+        size        => sub { _size      $_[1], $_[2][0], $_[2][1] },
+        # Add extra user filters
+        %{$conf->{filters}},
+    };
+
     # Get or set type
     $app->helper(vtype => sub {
         my ($self, $name, %opts) = @_;
-        return $conf->{types}{$name} = \%opts if %opts;
+        return $conf->{types}{$name} = \%opts if @_ > 2;
         return $conf->{types}{$name};
     });
 
+    # Get or set filter
+    $app->helper(vfilter => sub {
+        my ($self, $name, $sub) = @_;
+        return $conf->{filters}{$name} = $sub if @_ > 2;
+        return $conf->{filters}{$name};
+    });
+
     # Get or set config parameters
     $app->helper(vconf => sub {
         my ($self, $name, $value) = @_;
@@ -1076,6 +1460,39 @@ sub register {
         }
     });
 
+    # Return string if parameter have error, else empty string.
+    $app->helper(vclass => sub{
+        my ($self, $name, @classes) = @_;
+        return '' unless $self->verror( $name );
+
+        my @class;
+        push @class, $conf->{class}
+            if defined($conf->{class}) && length($conf->{class});
+        push @class, @classes;
+
+        return join ' ', @class;
+    });
+
+    $app->helper(vvalue => sub{
+        my ($self, $name, $default) = @_;
+
+        my @input = $self->can('every_param')
+            ? @{ $self->every_param( $name ) }
+            : $self->param( $name )
+        ;
+
+        my $value;
+        if( not @input ) {
+            $value = $default;
+        } elsif( @input > 1 ) {
+            $value = \@input;
+        } else {
+            $value = $input[0];
+        }
+
+        return $value;
+    });
+
     # Return all errors as Hash or errors count in scalar context.
     $app->helper(verrors => sub{
         my ($self) = @_;
@@ -1096,6 +1513,9 @@ sub register {
             : $conf->{optional}
         ;
 
+        my ($json, $pointer);
+        my ($dom,  $css);
+
         for my $name (keys %params) {
             # Param definition
             my $def = $params{$name};
@@ -1105,15 +1525,26 @@ sub register {
             if( 'HASH' eq ref $def ) {
                 %attr           = %$def;
             } elsif( 'Regexp' eq ref $def ) {
-                $attr{valid}    = sub { _like( $_[1], $def ) };
+                $attr{regexp}   = $def;
             } elsif( 'CODE' eq ref $def ) {
                 $attr{post}     = $def;
             } elsif( 'ARRAY' eq ref $def ) {
-                $attr{valid}    = sub { _in( $_[1], $def ) };
+                $attr{in}       = $def;
             } elsif( !ref $def ) {
                 $attr{type}     = $def;
             }
 
+            # Skip
+            if( exists $attr{skip} ) {
+                if( 'CODE' eq ref $attr{skip} ) {
+                    # Skip by sub result
+                    next if $attr{skip}->($self);
+                } elsif( $attr{skip} ) {
+                    # Skip by flag
+                    next;
+                }
+            }
+
             # Set default optional
             $attr{optional} = $optional unless defined $attr{optional};
 
@@ -1136,16 +1567,31 @@ sub register {
                         $attr{ $key } = $conf->{types}{ $type }{ $key };
                     }
                 } else {
-                    confess sprintf 'Type %s is not defined', $type;
+                    confess sprintf 'Type "%s" is not defined', $type;
                 }
             }
 
             # Get value
             my @input;
-            if( version->new($Mojolicious::VERSION) < version->new(5.28) ) {
-                @input = $self->param( $name );
+            if( $attr{jpath} ) {
+                # JSON Pointer
+                $json //= _parse_json( $self->req->body // '' );
+                if( $json ) {
+                    $pointer //= Mojo::JSON::Pointer->new( $json );
+                    @input = $pointer->get( $attr{jpath} );
+                }
+            } elsif( $attr{cpath} ) {
+                # CSS
+                $dom //= Mojo::DOM->new( $self->req->body // '' );
+                if( $dom ) {
+                    @input = $dom->find( $attr{cpath} )->map('text')->each;
+                }
             } else {
-                @input = @{ $self->every_param( $name ) };
+                # POST parameters
+                @input = $self->can('every_param')
+                    ? @{ $self->every_param( $name ) }
+                    : $self->param( $name )
+                ;
             }
 
             # Set undefined value if paremeter not set
@@ -1158,64 +1604,76 @@ sub register {
             # Process on all input values
             my @output;
             for my $index ( 0 .. $#input ) {
-                my $in  = my $out = $input[$index];
+                my $in = my $out = $input[$index];
 
-                if( defined $in ) {
-                    $out = $in;
+                $out = $in;
 
-                    # Apply pre filter
-                    $out = $attr{pre}->( $self, $out )    if $attr{pre};
+                # Apply pre filter
+                $out = $attr{pre}->( $self, $out )    if $attr{pre};
 
-                    # Apply validator
-                    if( $attr{valid} ) {
-                        if( my $error = $attr{valid}->($self, $out)  ) {
-                            # Set default value
-                            $out = $attr{default};
+                # Apply validator
+                if( $attr{valid} ) {
+                    if( my $error = $attr{valid}->($self, $out)  ) {
+                        # Set default value if error
+                        $out = $attr{default};
 
-                            # Default value always supress error
-                            $error = 0 if defined $attr{default};
-                            # Disable error on optional
-                            if( $attr{optional} ) {
-                                # Only if input param not set
-                                $error = 0 if not defined $in;
-                                $error = 0 if defined($in) and $in =~ m{^\s*$};
-                            }
-
-                            $self->verror(
-                                $name,
-                                %attr,
-                                index   => $index,
-                                in      => $in,
-                                out     => $out,
-                                message => $error,
-                            ) if $error;
+                        # Default value always supress error
+                        $error = 0 if defined $attr{default};
+                        # Disable error on optional
+                        if( $attr{optional} ) {
+                            # Only if input param not set
+                            $error = 0 if not defined $in;
+                            $error = 0 if defined($in) and $in =~ m{^\s*$};
                         }
+
+                        $self->verror(
+                            $name,
+                            %attr,
+                            index   => $index,
+                            in      => $in,
+                            out     => $out,
+                            message => $error,
+                        ) if $error;
                     }
+                }
 
-                    # Apply post filter
-                    $out = $attr{post}->( $self, $out )   if $attr{post};
+                # Hack for bool values:
+                # HTML forms do not transmit if checkbox off
+                $out = $attr{default}
+                    if $attr{type} && $attr{type} eq 'bool' and not defined $in;
 
-                    # Additional regexp
-                    if( defined $attr{regexp} ) {
-                        if( my $error = _like( $out, $attr{regexp}) ) {
-                            $out = $attr{default};
-                            # Default value always supress error
-                            $error = 0 if defined $attr{default};
+                # Apply post filter
+                $out = $attr{post}->( $self, $out )   if $attr{post};
 
-                            $self->verror(
-                                $name,
-                                %attr,
-                                index   => $index,
-                                in      => $in,
-                                out     => $out,
-                                message => $error,
-                            ) if $error;
+                for my $key ( keys %attr ) {
+                    # Skip unknown attribute
+                    next unless $conf->{filters}{ $key };
+
+                    my $error = $conf->{filters}{ $key }->(
+                        $self, $out, $attr{ $key }
+                    );
+                    if( $error ) {
+                        # Set default value if error
+                        $out = $attr{default};
+
+                        # Default value always supress error
+                        $error = 0 if defined $attr{default};
+                        # Disable error on optional
+                        if( $attr{optional} ) {
+                            # Only if input param not set
+                            $error = 0 if not defined $in;
+                            $error = 0 if defined($in) and $in =~ m{^\s*$};
                         }
-                    }
 
-                } else {
-                    $out = $attr{default};
-                    $out = $attr{post}->( $self, $out )   if $attr{post};
+                        $self->verror(
+                            $name,
+                            %attr,
+                            index   => $index,
+                            in      => $in,
+                            out     => $out,
+                            message => $error,
+                        ) if $error;
+                    }
                 }
 
                 # Запишем полученное значение
@@ -1227,7 +1685,7 @@ sub register {
                 if $attr{array} and not $attr{optional} and not @input;
 
             $result{ $name } = my $value = $attr{array} ? \@output : $output[0];
-            # Mojo::Validator::Validation
+            # Mojolicious::Validator::Validation
             $self->validation->output->{$name} = $value
                 if $conf->{mojo_validator};
         }
@@ -1254,8 +1712,7 @@ sub register {
         } elsif('CODE' eq ref $def) {
             $result = $self->vparams( $name => { post   => $def, %attr } );
         } elsif('ARRAY' eq ref $def) {
-            $def    = sub { _in( $_[1], $def ) };
-            $result = $self->vparams( $name => { valid  => $def, %attr } );
+            $result = $self->vparams( $name => { in     => $def, %attr } );
         } else {
             $result = $self->vparams( $name => { type   => $def, %attr } );
         }
@@ -1306,9 +1763,22 @@ sub register {
 
 =head1 RESTRICTIONS
 
-    * Version 1.0 invert L<valid> behavior: now checker return 0 if no error
-      or description string if has.
-    * New errors keys: orig => in, pre => out
+=over
+
+=item *
+
+Version 1.0 invert I<valid> behavior: now checker return 0 if no error
+or description string if has.
+
+=item *
+
+New errors keys: orig => in, pre => out
+
+=back
+
+=head1 SEE ALSO
+
+L<Mojolicious::Validator::Validation>, L<Mojolicious::Plugin::Human>.
 
 =head1 AUTHORS
 
